# auto generated by update_py.py

import os
import time
import threading
import signal
import zmq

import tlclient.linker.message_comm as message
from tlclient.linker.frame import Frame
from tlclient.linker.timer import Timer
from tlclient.linker.constant import MsgType, CommType, ANY_FIST_NAME
from tlclient.linker.utility import bytify


class Fist:

    def __init__(self, fist_name, fist_type, env_name, addr=None):
        self.fist_name = fist_name
        self.fist_type = fist_type
        self.env_name = env_name
        self._context = zmq.Context()
        # internal
        self.source_id = None
        self.pub_sock = None
        self.pull_sock = None
        self.rep_sock = None
        self.sub_master_sock = None
        self.req_master_sock = None
        self.master_addr = None
        self.sig_num = None
        self.influx_db_client = None
        self.sub_poller = zmq.Poller()
        self.push_socks = {}
        self.req_socks = {}
        self.fist_req_addrs = {}
        if addr:
            self.set_master_addr(addr)

    def recv(self, socket, timeout=1000):
        poll = zmq.Poller()
        poll.register(socket, zmq.POLLIN)
        sockets = dict(poll.poll(timeout))
        if socket in sockets:
            return socket.recv()
        else:
            print('Error: failed to receive data')
            return None

    def set_master_addr(self, addr):
        self.master_addr = addr

    def req_master(self, frame):
        self.req_master_sock.send(frame.buf)
        ret = self.recv(self.req_master_sock)
        if ret is None:
            raise Exception("Timeout in req_master!")
        return Frame(ret)

    def create_fist(self):
        req = message.ReqFistCreate()
        req.fist_name = bytify(self.fist_name)
        req.fist_type = self.fist_type
        req.env_name = bytify(self.env_name)
        req.pid = os.getpid()
        self.reg_req_master()
        f = Frame()
        f.set_msg_type(MsgType.REQ_FIST_CREATE)
        f.set_nano(Timer.nano())
        f.set_data(req)
        ret = self.req_master(f)
        ret_obj = ret.get_obj(message.RspFistCreate)
        if ret_obj.source <= 0:
            raise Exception("failed to create fist: {}".format(ret_obj.err_msg))
        self.source_id = ret_obj.source
        print('source_id', self.source_id)
        self.reg_sub_master()

    def set_pub(self):
        addr = self.get_bind_addr(CommType.Zmq_PUB)
        self.pub_sock = self._context.socket(zmq.PUB)
        self.pub_sock.bind(addr)
        print('pub', addr)
        return True

    def set_pull(self):
        addr = self.get_bind_addr(CommType.Zmq_PULL)
        self.pull_sock = self._context.socket(zmq.SUB)
        self.pull_sock.setsockopt(zmq.SUBSCRIBE, b'')
        self.pull_sock.bind(addr)
        print('pull', addr)
        return True

    def set_rep(self):
        addr = self.get_bind_addr(CommType.Zmq_REP)
        self.rep_sock = self._context.socket(zmq.REP)
        self.rep_sock.bind(addr)
        print('rep', addr)
        return True

    def reg_sub(self, other_fist_name):
        addr = self.get_connect_addr(other_fist_name, CommType.Zmq_PUB)
        sub_sock = self._context.socket(zmq.SUB)
        sub_sock.connect(addr)
        sub_sock.setsockopt(zmq.SUBSCRIBE, b'')
        self.sub_poller.register(sub_sock, zmq.POLLIN)
        print('sub', other_fist_name, addr)
        return True

    def reg_sub_master(self):
        addr = self.get_connect_addr('master', CommType.Zmq_PUB)
        self.sub_master_sock = self._context.socket(zmq.SUB)
        self.sub_master_sock.connect(addr)
        self.sub_master_sock.setsockopt(zmq.SUBSCRIBE, b'')
        print('sub_master', addr)
        return True

    def reg_req_master(self):
        self.req_master_sock = self._context.socket(zmq.REQ)
        self.req_master_sock.setsockopt(zmq.ROUTING_ID, bytify(self.fist_name))
        self.req_master_sock.connect(self.master_addr)
        print('req_master', self.master_addr)
        return True

    def reg_push(self, other_fist_name):
        addr = self.get_connect_addr(other_fist_name, CommType.Zmq_PULL)
        push_sock = self._context.socket(zmq.PUB)
        push_sock.connect(addr)
        self.push_socks[other_fist_name] = push_sock
        print('push', other_fist_name, addr)
        return True

    def reg_req(self, other_fist_name):
        addr = self.get_connect_addr(other_fist_name, CommType.Zmq_REP)
        assert addr is not None, 'fist req addr is None'
        self.fist_req_addrs[other_fist_name] = addr
        req_sock = self._context.socket(zmq.REQ)
        req_sock.connect(addr)
        self.req_socks[other_fist_name] = req_sock
        print('req', other_fist_name, addr)
        return True

    def pub_f(self, frame):
        self.pub_sock.send(frame.buf)

    def pub(self, obj, msg_type, req_id, err_id=0):
        f = Frame()
        f.set_msg_type(msg_type)
        f.set_req_id(req_id)
        f.set_err_id(err_id)
        f.set_nano(Timer.nano())
        f.set_source(self.get_source_id())
        f.set_data(obj)
        self.pub_f(f)

    def push(self, fist_name, obj, msg_type, req_id, err_id=0):
        ps_sock = self.push_socks[fist_name]
        f = Frame()
        f.set_msg_type(msg_type)
        f.set_req_id(req_id)
        f.set_err_id(err_id)
        f.set_nano(Timer.nano())
        f.set_source(self.get_source_id())
        f.set_data(obj)
        ps_sock.send(f.buf)

    def req(self, fist_name, obj, msg_type, req_id, err_id=0):
        req_sock = self.req_socks[fist_name]
        f = Frame()
        f.set_msg_type(msg_type)
        f.set_req_id(req_id)
        f.set_err_id(err_id)
        f.set_nano(Timer.nano())
        f.set_source(self.get_source_id())
        f.set_data(obj)
        try:
            req_sock.send(f.buf)
        except zmq.error.ZMQError as e:
            # double req problem, reset req socket
            if e.errno == 156384763:
                req_sock = self._context.socket(zmq.REQ)
                req_sock.connect(self.fist_req_addrs[fist_name])
                req_sock.send(f.buf)
                self.req_socks[fist_name] = req_sock
            # all other errs, reraise err
            else:
                raise e
        ret = self.recv(req_sock)
        if ret is None:
            raise Exception("Timeout in req {}!".format(fist_name))
        return Frame(ret)

    def notify(self, title, content, notification_type):
        req = message.ReqNotify()
        req.title = bytify(title)
        req.message = bytify(content)
        req.type = notification_type
        f = Frame()
        f.set_msg_type(MsgType.REQ_NOTIFY)
        f.set_source(self.get_source_id())
        f.set_nano(Timer.nano())
        f.set_data(req)
        ret = self.req_master(f)
        rsp_obj = ret.get_obj(message.RspNotify)
        if not rsp_obj.accepted:
            print('[notify] failed, (err_msg){}'.format(rsp_obj.err_msg))
        return rsp_obj.accepted

    def on_pub_frame(self, f):
        pass

    def on_push_frame(self, f):
        pass

    def on_req_frame(self, f):
        return Frame()

    def on_pub_master(self, f):
        if f.get_msg_type() == MsgType.CMD_SUICIDE:
            suicide_req = f.get_obj(message.ReqFistSuicide)
            fist_name = suicide_req.fist_name.decode()
            if fist_name in [ANY_FIST_NAME, self.fist_name]:
                self.stop()

    def on_close(self):
        pass

    def start(self):
        signal.signal(signal.SIGTERM, self.signal_handler)
        signal.signal(signal.SIGINT, self.signal_handler)
        ts = [
            threading.Thread(target=self.run_sub),
            threading.Thread(target=self.run_pull),
            threading.Thread(target=self.run_rep),
            threading.Thread(target=self.run_sub_master),
        ]
        for t in ts:
            t.setDaemon(True)
            t.start()

    def stop(self, signum=signal.SIGTERM):
        self.on_close()
        self.sig_num = signum

    def join(self):
        while True:
            time.sleep(0.01)
            if self.sig_num is not None:
                print('[main_thread] ended (sig){}'.format(self.sig_num))
                break

    def signal_handler(self, signum=None, frame=None):
        print('[signal] received signal, to stop... (sig){}'.format(signum))
        self.stop(signum=signum)

    def get_source_id(self):
        return self.source_id

    ######################
    # internal functions #
    ######################
    def get_bind_addr(self, comm_type):
        req = message.ReqFistSet()
        req.comm_type = comm_type
        f = Frame()
        f.set_msg_type(MsgType.REQ_FIST_SET)
        f.set_source(self.get_source_id())
        f.set_nano(Timer.nano())
        f.set_data(req)
        ret = self.req_master(f)
        rsp_obj = ret.get_obj(message.RspFistSet)
        if rsp_obj.is_allowed:
            return rsp_obj.addr
        else:
            raise Exception("Rejected: (fist_name){} (comm_type){} (err_msg){}".format(self.fist_name, comm_type, rsp_obj.err_msg))

    def get_connect_addr(self, fist_name, comm_type):
        req = message.ReqFistReg()
        req.fist_name = bytify(fist_name)
        req.comm_type = comm_type
        f = Frame()
        f.set_msg_type(MsgType.REQ_FIST_REG)
        f.set_source(self.get_source_id())
        f.set_nano(Timer.nano())
        f.set_data(req)
        ret = self.req_master(f)
        rsp_obj = ret.get_obj(message.RspFistReg)
        # WARNING!! DO NOT use the addr if it's not connectable
        if not rsp_obj.input_registered or not rsp_obj.input_connectable:
            print('[get_connect_addr] rejected by master (fist_name){} (comm_type){} (reged){} (connectable){}'.format(fist_name, comm_type, rsp_obj.input_registered, rsp_obj.input_connectable))
            return None
        else:
            return rsp_obj.addr

    def run_sub(self):
        while self.sig_num is None:
            socks = dict(self.sub_poller.poll(1))
            for sock in socks:
                ret = sock.recv()
                f = Frame(ret)
                self.on_pub_frame(f)
        print('[sub_thread] ended (sig){}'.format(self.sig_num))

    def run_sub_master(self):
        if self.sub_master_sock:
            while self.sig_num is None:
                ret = self.sub_master_sock.recv()
                f = Frame(ret)
                self.on_pub_master(f)
            print('[sub_master_thread] ended (sig){}'.format(self.sig_num))

    def run_pull(self):
        if self.pull_sock:
            while self.sig_num is None:
                ret = self.pull_sock.recv()
                f = Frame(ret)
                self.on_push_frame(f)
            print('[pull_thread] ended (sig){}'.format(self.sig_num))

    def run_rep(self):
        if self.rep_sock:
            while self.sig_num is None:
                ret = self.rep_sock.recv()
                f = Frame(ret)
                rsp_f = self.on_req_frame(f)
                self.rep_sock.send(rsp_f.buf)
            print('[rep_thread] ended (sig){}'.format(self.sig_num))

    # db setting
    def set_influxdb(self, host='localhost', port=8086, user='', passwd='', db='traders_link'):
        from influxdb import InfluxDBClient
        self.influx_db_client = InfluxDBClient(host, port, user, passwd, db)

    def write_points(self, points):
        if self.influx_db_client is None:
            return False
        if not isinstance(points, list):
            points = [points]
        print(points)
        return self.influx_db_client.write_points(points)

    def query(self, sql):
        if self.influx_db_client is None:
            return None
        return self.influx_db_client.query(sql)
