# auto generated by update_py.py

import ctypes
from tlclient.linker.timer import Timer
from tlclient.trader.constant import (MODULE_TICKER_LENGTH, MODULE_TRADINGDAY_LENGTH,
                             ORDER_BOOK_SIZE, ExchangeID, MODULE_ERR_MSG_LENGTH)


class MktBasic(ctypes.Structure):
    _fields_ = [
        ('exchange', ctypes.c_short),
        ('ticker', ctypes.c_char * MODULE_TICKER_LENGTH),
        ('mkt_time', ctypes.c_int64),
        ('trading_day', ctypes.c_char * MODULE_TRADINGDAY_LENGTH)
    ]

    def to_dict(self):
        return {
            'exchange': self.exchange,
            'ticker': self.ticker.decode(),
            'mkt_time': self.mkt_time,
            'trading_day': self.trading_day.decode(),
        }

    def to_influx(self):
        return {
            "measurement": "MktBasic",
            "tags": {
                "exchange": self.exchange,
                "ticker": self.ticker,
                "trading_day": self.trading_day
            },
            "time": Timer.nano_to_datetime(self.mkt_time),
            "fields": {
            }
        }

class MktSnap(MktBasic):
    _fields_ = [
        ('ask_price', ctypes.c_double * ORDER_BOOK_SIZE),
        ('bid_price', ctypes.c_double * ORDER_BOOK_SIZE),
        ('ask_volume', ctypes.c_int64 * ORDER_BOOK_SIZE),
        ('bid_volume', ctypes.c_int64 * ORDER_BOOK_SIZE)
    ]

    def __str__(self):
        return '<snap-{}-{}ask1={}@{} bid1={}@{}>'.format(
            ExchangeID.read(self.exchange), self.ticker,
            self.ask_volume[0], self.ask_price[0],
            self.bid_volume[0], self.bid_price[0],
        )

    def to_dict(self):
        return dict(MktBasic.to_dict(self), **{
            'ask_prices': list(self.ask_price),
            'bid_prices': list(self.bid_price),
            'ask_volumes': list(self.ask_volume),
            'bid_volumes': list(self.bid_volume),
        })

    def to_influx(self):
        return {
            "measurement": "MktSnap",
            "tags": MktBasic.to_dict(self)['tags'],
            "time": Timer.nano_to_datetime(self.mkt_time),
            "fields": dict(MktBasic.to_dict(self)['fields'], **{
                "ask_price1": self.ask_price[0],
                "ask_price2": self.ask_price[1],
                "ask_price3": self.ask_price[2],
                "ask_price4": self.ask_price[3],
                "ask_price5": self.ask_price[4],
                "ask_volume1": self.ask_volume[0],
                "ask_volume2": self.ask_volume[1],
                "ask_volume3": self.ask_volume[2],
                "ask_volume4": self.ask_volume[3],
                "ask_volume5": self.ask_volume[4],
                "bid_price1": self.bid_price[0],
                "bid_price2": self.bid_price[1],
                "bid_price3": self.bid_price[2],
                "bid_price4": self.bid_price[3],
                "bid_price5": self.bid_price[4],
                "bid_volume1": self.bid_volume[0],
                "bid_volume2": self.bid_volume[1],
                "bid_volume3": self.bid_volume[2],
                "bid_volume4": self.bid_volume[3],
                "bid_volume5": self.bid_volume[4],
            })
        }

class MktSnapPlus(MktSnap):
    _fields_ = [
        ('upper_limit_price', ctypes.c_double),
        ('lower_limit_price', ctypes.c_double),
        ('last_price', ctypes.c_double),
        ('pre_close_price', ctypes.c_double),
        ('open_price', ctypes.c_double),
        ('high_price', ctypes.c_double),
        ('low_price', ctypes.c_double),
        ('close_price', ctypes.c_double),
        ('volume', ctypes.c_int64),
        ('turnover', ctypes.c_double),
        ('trading_count', ctypes.c_int64),
    ]

    def __str__(self):
        return '<snap-{}-{} last_px={} vol={} ask1={}@{} bid1={}@{}>'.format(
            ExchangeID.read(self.exchange), self.ticker, self.last_price, self.volume,
            self.ask_volume[0], self.ask_price[0],
            self.bid_volume[0], self.bid_price[0],
        )

    def to_dict(self):
        return dict(MktSnap.to_dict(self), **{
            'upper_limit_price': self.upper_limit_price,
            'lower_limit_price': self.lower_limit_price,
            'last_price': self.last_price,
            'pre_close_price': self.pre_close_price,
            'open_price': self.open_price,
            'high_price': self.high_price,
            'low_price': self.low_price,
            'close_price': self.close_price,
            'volume': self.volume,
            'turnover': self.turnover,
            'trading_count': self.trading_count,
        })

    def to_influx(self):
        return {
            "measurement": "MktSnapPlus",
            "tags": MktSnap.to_dict(self)['tags'],
            "time": Timer.nano_to_datetime(self.mkt_time),
            "fields": dict(MktSnap.to_dict(self)['fields'], **{
                "upper_limit_price": self.upper_limit_price,
                "lower_limit_price": self.lower_limit_price,
                "last_price": self.last_price,
                "pre_close_price": self.pre_close_price,
                "open_price": self.open_price,
                "high_price": self.high_price,
                "low_price": self.low_price,
                "close_price": self.close_price,
                "volume": self.volume,
                "turnover": self.turnover,
                "trading_count": self.trading_count,
            })
        }

class MktSnapFut(MktSnapPlus):
    _fields_ = [
        ('pre_open_interest', ctypes.c_double),
        ('open_interest', ctypes.c_double),
        ('pre_settlement_price', ctypes.c_double),
        ('settlement_price', ctypes.c_double)
    ]

    def to_dict(self):
        return dict(MktSnapPlus.to_dict(self), **{
            'pre_open_interest': self.pre_open_interest,
            'open_interest': self.open_interest,
            'pre_settlement_price': self.pre_settlement_price,
            'settlement_price': self.settlement_price
        })

    def to_influx(self):
        return {
            "measurement": "MktSnapFut",
            "tags": MktSnapPlus.to_dict(self)['tags'],
            "time": Timer.nano_to_datetime(self.mkt_time),
            "fields": dict(MktSnapPlus.to_dict(self)['fields'], **{
                "pre_open_interest": self.pre_open_interest,
                "open_interest": self.open_interest,
                "pre_settlement_price": self.pre_settlement_price,
                "settlement_price": self.settlement_price,
            })
        }

class MktSnapOpt(MktSnapFut):
    _fields_ = [
        ('pre_delta', ctypes.c_double),
        ('curr_delta', ctypes.c_double),
    ]

    def to_dict(self):
        return dict(MktSnapFut.to_dict(self), **{
            'pre_delta': self.pre_delta,
            'curr_delta': self.curr_delta
        })

    def to_influx(self):
        return {
            "measurement": "MktSnapOpt",
            "tags": MktSnapFut.to_dict(self)['tags'],
            "time": Timer.nano_to_datetime(self.mkt_time),
            "fields": dict(MktSnapFut.to_dict(self)['fields'], **{
                "pre_delta": self.pre_delta,
                "curr_delta": self.curr_delta
            })
        }

class MktIndex(MktBasic):
    _fields_ = [
        ('last', ctypes.c_double),
        ('pre_close', ctypes.c_double),
        ('open', ctypes.c_double),
        ('high', ctypes.c_double),
        ('low', ctypes.c_double),
        ('close', ctypes.c_double),
        ('volume', ctypes.c_int64),
        ('turnover', ctypes.c_double),
    ]

    def to_dict(self):
        return dict(MktBasic.to_dict(self), **{
            'last': self.last,
            'pre_close': self.pre_close,
            'open': self.open,
            'high': self.high,
            'low': self.low,
            'close': self.close,
            'volume': self.volume,
            'turnover': self.turnover
        })

    def to_influx(self):
        return {
            "measurement": "MktIndex",
            "tags": MktBasic.to_dict(self)['tags'],
            "time": Timer.nano_to_datetime(self.mkt_time),
            "fields": dict(MktBasic.to_dict(self)['fields'], **{
                'last': self.last,
                'pre_close': self.pre_close,
                'open': self.open,
                'high': self.high,
                'low': self.low,
                'close': self.close,
                'volume': self.volume,
                'turnover': self.turnover
            })
        }

class MktOrder(MktBasic):
    _fields_ = [
        ('group_id', ctypes.c_int64),
        ('index', ctypes.c_int64),
        ('price', ctypes.c_double),
        ('volume', ctypes.c_int64),
        ('order_type', ctypes.c_char),
    ]

    def __str__(self):
        return '<md_order-{}-{} mkt_time={} group_id={} index={} price={} volume={} order_type={}>'.format(
            ExchangeID.read(self.exchange), self.ticker, Timer.nano_to_datetime(self.mkt_time),
            self.group_id, self.index,
            self.price, self.volume,
            self.order_type
        )

    def to_dict(self):
        return dict(MktBasic.to_dict(self), **{
            'group_id': self.group_id,
            'index': self.index,
            'price': self.price,
            'volume': self.volume,
            'order_type': self.order_type
        })

    def to_influx(self):
        return {
            "measurement": "MktOrder",
            "tags": MktBasic.to_dict(self)['tags'],
            "time": Timer.nano_to_datetime(self.mkt_time),
            "fields": dict(MktBasic.to_dict(self)['fields'], **{
                'group_id': self.group_id,
                'index': self.index,
                'price': self.price,
                'volume': self.volume,
                'order_type': self.order_type
            })
        }

class MktTrade(MktBasic):
    _fields_ = [
        ('group_id', ctypes.c_int64),
        ('index', ctypes.c_int64),
        ('price', ctypes.c_double),
        ('volume', ctypes.c_int64),
        ('order_type', ctypes.c_char),
        ('buy_order_index', ctypes.c_int64),
        ('sell_order_index', ctypes.c_int64),
    ]

    def __str__(self):
        return '<md_trade-{}-{} mkt_time={} group_id={} index={} price={} volume={} order_type={} buy_order={} sell_order={}>'.format(
            ExchangeID.read(self.exchange), self.ticker, Timer.nano_to_datetime(self.mkt_time),
            self.group_id, self.index,
            self.price, self.volume,
            self.order_type,
            self.buy_order_index, self.sell_order_index
        )

    def to_dict(self):
        return dict(MktBasic.to_dict(self), **{
            'group_id': self.group_id,
            'index': self.index,
            'price': self.price,
            'volume': self.volume,
            'order_type': self.order_type,
            'buy_order_index': self.buy_order_index,
            'sell_order_index': self.sell_order_index
        })

    def to_influx(self):
        return {
            "measurement": "MktTrade",
            "tags": MktBasic.to_dict(self)['tags'],
            "time": Timer.nano_to_datetime(self.mkt_time),
            "fields": dict(MktBasic.to_dict(self)['fields'], **{
                'group_id': self.group_id,
                'index': self.index,
                'price': self.price,
                'volume': self.volume,
                'order_type': self.order_type,
                'buy_order_index': self.buy_order_index,
                'sell_order_index': self.sell_order_index
            })
        }

class MktBar(MktBasic):
    _fields_ = [
        ('sec_interval', ctypes.c_int),
        ('open', ctypes.c_double),
        ('close', ctypes.c_double),
        ('low', ctypes.c_double),
        ('high', ctypes.c_double),
        ('volume', ctypes.c_int64),
    ]

    def __str__(self):
        return '<bar-{}-{} O={} C={} L={} H={} V={}>'.format(
            ExchangeID.read(self.exchange), self.ticker, 
            self.open, self.close, self.low, self.high, self.volume,
        )

    def to_dict(self):
        return dict(MktBasic.to_dict(self), **{
            'sec_interval': self.sec_interval,
            'open': self.open,
            'close': self.close,
            'low': self.low,
            'high': self.high,
            'volume': self.volume
        })

    def to_influx(self):
        return {
            "measurement": "MktBar",
            "tags": dict(MktBasic.to_dict(self)['tags'], **{
                'sec_interval': self.sec_interval
            }),
            "time": Timer.nano_to_datetime(self.mkt_time),
            "fields": dict(MktBasic.to_dict(self)['fields'], **{
                'open': self.open,
                'close': self.close,
                'low': self.low,
                'high': self.high,
                'volume': self.volume
            })
        }

class MktBarGen(MktBar):
    _fields_ = [
        ('start_nano', ctypes.c_int64),
        ('end_nano', ctypes.c_int64),
        ('start_volume', ctypes.c_int64),
    ]

    def to_dict(self):
        return dict(MktBar.to_dict(self), **{
            'start_nano': self.start_nano,
            'end_nano': self.end_nano,
            'start_volume': self.start_volume
        })

    def to_influx(self):
        return {
            "measurement": "MktBarGen",
            "tags": dict(MktBar.to_dict(self)['tags'], **{
                'start_nano': self.start_nano,
                'end_nano': self.end_nano,
                'start_volume': self.start_volume,
            }),
            "time": Timer.nano_to_datetime(self.mkt_time),
            "fields": MktBar.to_dict(self)['fields']
        }
